import { system, world } from '@minecraft/server';


// Función para invocar entidades alrededor con cierto rango
function spawnEntitiesAround(entity, count, entityType, range) {
    const location = entity.location;
    const dimension = entity.dimension;

    for (let i = 0; i < count; i++) {
        const offsetX = (Math.random() * 2 - 1) * range; // Valor entre -range y range
        const offsetZ = (Math.random() * 2 - 1) * range; // Valor entre -range y range
        const spawnLocation = {
            x: location.x + offsetX,
            y: location.y,
            z: location.z + offsetZ
        };

        dimension.spawnEntity(entityType, spawnLocation);
    }
}


world.afterEvents.itemUse.subscribe(ev => {
    let item = ev.itemStack
    let source = ev.source
    if (item.typeId == 'eu:crown_king_slime') {
        source.dimension.spawnEntity('eu:king_slime', source.location)
        source.getComponent('equippable').setEquipment('Mainhand', undefined)
    }
})

const slimeHealthTracker = new Map();
world.afterEvents.entityHealthChanged.subscribe(ev => {
    const entity = ev.entity;
    const healthOld = ev.oldValue;
    const healthNew = ev.newValue;

    if (entity.typeId === 'eu:king_slime') {
        let healthLost = slimeHealthTracker.get(entity.id) || 0;
        healthLost += healthOld - healthNew;

        if (healthLost >= 100) {
            // Resetea el contador de salud perdida
            healthLost = 0;
            // Invoca slimes alrededor del "king slime"
            spawnEntitiesAround(entity, 5, 'minecraft:slime', 3); // Invoca 5 slimes alrededor del king slime en un rango de 3 bloques
        }

        // Actualiza el registro de salud perdida
        slimeHealthTracker.set(entity.id, healthLost);
    }
});



system.runInterval(() => {
    for (const player of world.getPlayers()) {
        const kingSlimes = player.dimension.getEntities({
            type: "eu:king_slime",
            location: player.location,
            maxDistance: 25
        });

        kingSlimes.forEach(slime => {
            //const location = slime.location;
            if (slime.isInWater && !player.isInWater) {
                slime.playAnimation('animation.teleport_slime.new')
                system.runTimeout(() => {
                    slime?.runCommand('tp @s @p')
                }, 1.5 * 20)
            }

        });
    }
}, 20 * 3);


///al entrar en agua salta y le aplica un aplaston al jugador mas cercano
world.afterEvents.entitySpawn.subscribe(ev => {
    let entity = ev.entity
    if (entity.typeId == 'eu:king_slime') {
        entity.runCommand('weather thunder')
        world.sendMessage('§bKing slime has arrived!§r')
    }

})
world.afterEvents.entityHurt.subscribe(ev => {
    const hurtEntity = ev.hurtEntity;

    if (hurtEntity.typeId === 'eu:king_slime' && !hurtEntity.hasTag('jump_attack')) {
        const targetHurtEntity = hurtEntity.target;
        if (targetHurtEntity?.location) {
            let targetLocation = targetHurtEntity.location;
            let hurtEntityLocation = hurtEntity.location;

            let dx = targetLocation.x - hurtEntityLocation.x;
            let dy = targetLocation.y - hurtEntityLocation.y;
            let dz = targetLocation.z - hurtEntityLocation.z;

            const random = Math.random();
            const probability = 0.2;

            if (random < probability) {
                hurtEntity.addTag('jump_attack');
                hurtEntity.applyImpulse({ x: dx / 8, y: 3, z: dz / 8 });

                system.runTimeout(() => {
                    // Obtener las nuevas ubicaciones del objetivo y de la entidad herida
                    targetLocation = targetHurtEntity.location;
                    hurtEntityLocation = hurtEntity.location;

                    // Calcular las nuevas diferencias de distancia
                    dx = targetLocation.x - hurtEntityLocation.x;
                    dy = targetLocation.y - hurtEntityLocation.y;
                    dz = targetLocation.z - hurtEntityLocation.z;
                    hurtEntity.addEffect('strength', 5 * 20, { amplifier: 1, showParticles: true })
                    hurtEntity.applyImpulse({ x: dx / 8, y: -3, z: dz / 8 });
                    hurtEntity.removeTag('jump_attack');
                }, 2 * 20); // 2 * 20 ticks (2 segundos)
            }
        }
    }
});

